CREATE PROC [Reporting].[PowerPlan] AS

DECLARE @DATEFUNCTION DATE = GETDATE()

DROP TABLE [Reporting].[PowerPlan_BASE];

DECLARE @deadlock_var NCHAR(3);
SET @deadlock_var = N'LOW';

BEGIN
	SET NOCOUNT ON
	SET DEADLOCK_PRIORITY LOW
	SET DEADLOCK_PRIORITY NORMAL
	SET DEADLOCK_PRIORITY HIGH
	SET DEADLOCK_PRIORITY @deadlock_var
	SET DEADLOCK_PRIORITY 10
	SET DEADLOCK_PRIORITY -5
	SELECT 1

	CREATE TABLE #TempTest
	WITH (DISTRIBUTION = ROUND_ROBIN, HEAP) AS
	(SELECT 2 AS Two,
			3 AS Three,
			4 AS Four
	)
END
