SET TRANSACTION ISOLATION LEVEL REPEATABLE READ;
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;
SET GLOBAL TRANSACTION ISOLATION LEVEL REPEATABLE READ;
SET SESSION TRANSACTION ISOLATION LEVEL READ COMMITTED;
SET TRANSACTION READ WRITE;
SET GLOBAL TRANSACTION READ ONLY;
SET SESSION TRANSACTION ISOLATION LEVEL REPEATABLE READ, READ ONLY;
SET GLOBAL TRANSACTION ISOLATION LEVEL SERIALIZABLE, READ WRITE;
SET SESSION TRANSACTION READ WRITE, ISOLATION LEVEL READ UNCOMMITTED;
SET TRANSACTION READ ONLY, ISOLATION LEVEL SERIALIZABLE;
