CREATE STREAMLIT new_streamlit
ROOT_LOCATION = '@stage_name/folder'
MAIN_FILE = 'main.py';

CREATE OR REPLACE STREAMLIT new_streamlit
ROOT_LOCATION = '@stage_name/folder'
MAIN_FILE = 'main.py';

CREATE STREAMLIT new_streamlit
ROOT_LOCATION = '@stage_name/folder'
MAIN_FILE = 'main.py'
QUERY_WAREHOUSE = my_wh;

CREATE OR REPLACE STREAMLIT new_streamlit
ROOT_LOCATION = '@stage_name/folder'
MAIN_FILE = 'main.py'
QUERY_WAREHOUSE = my_wh;

CREATE STREAMLIT new_streamlit
ROOT_LOCATION = '@stage_name/folder'
MAIN_FILE = 'main.py'
QUERY_WAREHOUSE = my_wh
COMMENT = 'amazing comment';

CREATE OR REPLACE STREAMLIT new_streamlit
ROOT_LOCATION = '@stage_name/folder'
MAIN_FILE = 'main.py'
QUERY_WAREHOUSE = my_wh
COMMENT = 'amazing comment';
