USE database_name;

-- Use the 'userdb'
USE userdb;

-- Use the 'userdb1'
USE userdb1;

-- Keywords SCHEMA and DATABASE are interchangeable.
USE DATABASE database_name;

USE SCHEMA database_name;

USE IDENTIFIER('database_name');

USE DATABASE IDENTIFIER('database_name');

USE SCHEMA IDENTIFIER('database_name');
