CLUSTER employees USING employees_ind;
CLUSTER VERBOSE employees USING employees_ind;
CLUSTER employees;
CLUSTER;
CLUSTER VERBOSE;
CLUSTER index_name ON table_name;
CLUSTER public.temp_table USING idx_temp_table_ra;
