SELECT '12'::Nullable(Decimal(12,0)) as num_value;

SELECT '12'::Nullable(Decimal32(1)) as num_value;

SELECT '12'::Nullable(Decimal64(12)) as num_value;

SELECT '12'::Nullable(Decimal128(12)) as num_value;

SELECT '12'::Nullable(Decimal256(12)) as num_value;

SELECT '12'::Decimal(12,0) as num_value;

SELECT '12'::Nullable(Numeric(12, 0)) as num_value;
