-- https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical?hl=en

SELECT NUMERIC '0';

SELECT NUMERIC '0' = NUMERIC '0';

SELECT NUMERIC "0";

SELECT NUMERIC "0" = NUMERIC "0";

SELECT BIGNUMERIC '0';

SELECT BIGNUMERIC '0' = BIGNUMERIC '0';

SELECT BIGNUMERIC "0";

SELECT BIGNUMERIC "0" = BIGNUMERIC "0";

SELECT DATE '2014-09-27';

SELECT DATE '2014-09-27' = DATE '2014-09-27';

SELECT DATE "2014-09-27";

SELECT DATE "2014-09-27" = DATE "2014-09-27";

SELECT TIME '12:30:00.45';

SELECT TIME '12:30:00.45' = TIME '12:30:00.45';

SELECT TIME "12:30:00.45";

SELECT TIME "12:30:00.45" = TIME "12:30:00.45";

SELECT DATETIME '2014-09-27 12:30:00.45';

SELECT DATETIME '2014-09-27 12:30:00.45' = DATETIME '2014-09-27 12:30:00.45';

SELECT DATETIME "2014-09-27 12:30:00.45";

SELECT DATETIME "2014-09-27 12:30:00.45" = DATETIME "2014-09-27 12:30:00.45";

SELECT TIMESTAMP '2014-09-27 12:30:00.45-08';

SELECT TIMESTAMP '2014-09-27 12:30:00.45-08' = TIMESTAMP '2014-09-27 12:30:00.45-08';

SELECT TIMESTAMP "2014-09-27 12:30:00.45-08";

SELECT TIMESTAMP "2014-09-27 12:30:00.45-08" = TIMESTAMP "2014-09-27 12:30:00.45-08";

SELECT INTERVAL '10:20:30.52' HOUR TO SECOND;

SELECT INTERVAL '10:20:30.52' HOUR TO SECOND = INTERVAL '10:20:30.52' HOUR TO SECOND;

SELECT INTERVAL "10:20:30.52" HOUR TO SECOND;

SELECT INTERVAL "10:20:30.52" HOUR TO SECOND = INTERVAL "10:20:30.52" HOUR TO SECOND;

SELECT JSON '{}';

SELECT JSON '{}' IS NOT NULL;

SELECT JSON "{}";

SELECT JSON "{}" IS NOT NULL;
