CREATE DATABASE my_database;

CREATE DATABASE IF NOT EXISTS my_database;

CREATE DATABASE my_database
DEFAULT CHARACTER SET utf8mb4
COLLATE utf8mb4_0900_ai_ci
DEFAULT ENCRYPTION 'N';

CREATE DATABASE my_database
DEFAULT CHARACTER SET = utf8mb4
COLLATE = utf8mb4_0900_ai_ci
DEFAULT ENCRYPTION = 'N';

CREATE SCHEMA my_database
DEFAULT CHARACTER SET utf8mb4
COLLATE utf8mb4_0900_ai_ci
DEFAULT ENCRYPTION 'N';

CREATE DATABASE IF NOT EXISTS xxx CHARACTER SET "utf8mb4" COLLATE "utf8mb4_bin";
