-- BASIC CREATE VIEW
CREATE VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE OR REPLACE
CREATE OR REPLACE VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE VIEW WITH FORCE
CREATE FORCE VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE VIEW WITH NO FORCE
CREATE NO FORCE VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);
CREATE OR REPLACE NO FORCE VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE VIEW WITH EDITIONING
CREATE EDITIONING VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE VIEW WITH EDITIONABLE
CREATE EDITIONABLE VIEW  NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE VIEW WITH NONEDITIONABLE
CREATE NONEDITIONABLE VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE VIEW WITH EDITIONABLE EDITIONING
CREATE EDITIONABLE EDITIONING VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE OR REPLACE VIEW WITH EDITIONING
CREATE OR REPLACE EDITIONING VIEW NEW_VIEW AS (select PERSON_ID from PERSONS p);

-- CREATE MATERIALIZED VIEW
CREATE MATERIALIZED VIEW NEW_VIEW AS( select PERSON_ID from PERSONS p);

-- CREATE OR REPLACE MATERIALIZED VIEW
CREATE OR REPLACE MATERIALIZED VIEW NEW_VIEW AS( select PERSON_ID from PERSONS p);
