ALTER DATABASE my_database
DEFAULT CHARACTER SET utf8mb4
COLLATE utf8mb4_0900_ai_ci
DEFAULT ENCRYPTION 'N';

ALTER DATABASE my_database
DEFAULT CHARACTER SET = utf8mb4
COLLATE = utf8mb4_0900_ai_ci
DEFAULT ENCRYPTION = 'N';

ALTER SCHEMA my_database
DEFAULT CHARACTER SET utf8mb4
COLLATE utf8mb4_0900_ai_ci
DEFAULT ENCRYPTION 'N';

ALTER DATABASE my_database
READ ONLY DEFAULT;

ALTER DATABASE my_database
READ ONLY 0;

ALTER DATABASE my_database
READ ONLY 1;

ALTER DATABASE
READ ONLY DEFAULT;
