SELECT '2024-01-01'::DateTime64 as datetime;

SELECT '2024-01-01'::DateTime64() as datetime;

SELECT '2024-01-01'::DateTime64(3) as datetime;

SELECT '2024-01-01'::DateTime64(3, 'Europe/Paris') as datetime;

SELECT '2024-01-01'::DateTime64(6) as datetime;
