SELECT * FROM MY_TABLE WHERE A >= B;

SELECT * FROM MY_TABLE WHERE A GE B;

SELECT * FROM MY_TABLE WHERE A <= B;

SELECT * FROM MY_TABLE WHERE A LE B;

SELECT * FROM MY_TABLE WHERE A = B;

SELECT * FROM MY_TABLE WHERE A EQ B;

SELECT * FROM MY_TABLE WHERE A <> B;

SELECT * FROM MY_TABLE WHERE A ^= B;

SELECT * FROM MY_TABLE WHERE A NOT= B;

SELECT * FROM MY_TABLE WHERE A NE B;

SELECT * FROM MY_TABLE WHERE A GT B;

SELECT * FROM MY_TABLE WHERE A > B;

SELECT * FROM MY_TABLE WHERE A LT B;

SELECT * FROM MY_TABLE WHERE A < B;
