SELECT EXTRACT(YEAR FROM '2019-07-02');
SELECT TIMESTAMPADD(MINUTE, 1, '2003-01-02');
SELECT TIMESTAMPDIFF(MONTH, '2003-02-01', '2003-05-01');
SELECT TIMESTAMPDIFF(HOUR, x.time_created, x.time_updated)
FROM example_table AS x;
SELECT TIMESTAMPADD(SECOND, 1, x.some_timestamp_field)
FROM example_table AS x;
