select column_name
from   table_name
fetch  first row only;

select column_name
from   table_name
fetch  first rows only;

select column_name
from   table_name
fetch  first 2 row only;

select column_name
from   table_name
fetch  first 2 rows only;
