VALUES (
    CURRENT_DATE,
    CURRENT_PATH,
    CURRENT_SCHEMA,
    CURRENT_SERVER,
    CURRENT_TIME,
    CURRENT_TIMESTAMP,
    CURRENT_TIMEZONE,
    CURRENT_USER,
    CURRENT CLIENT_ACCTNG,
    CURRENT CLIENT_APPLNAME,
    CURRENT CLIENT_USERID,
    CURRENT CLIENT_WRKSTNNAME,
    CURRENT DATE,
    CURRENT DBPARTITIONNUM,
    CURRENT DECFLOAT ROUNDING MODE,
    CURRENT DEFAULT TRANSFORM GROUP,
    CURRENT DEGREE,
    CURRENT EXPLAIN MODE,
    CURRENT EXPLAIN SNAPSHOT,
    CURRENT FEDERATED ASYNCHRONY,
    CURRENT IMPLICIT XMLPARSE OPTION,
    CURRENT ISOLATION,
    CURRENT LOCALE LC_MESSAGES,
    CURRENT LOCALE LC_TIME,
    CURRENT LOCK TIMEOUT,
    CURRENT MAINTAINED TABLE TYPES FOR OPTIMIZATION,
    CURRENT MDC ROLLOUT MODE,
    CURRENT MEMBER,
    CURRENT OPTIMIZATION PROFILE,
    CURRENT PACKAGE PATH,
    CURRENT PATH,
    CURRENT QUERY OPTIMIZATION,
    CURRENT REFRESH AGE,
    CURRENT SCHEMA,
    CURRENT SERVER,
    CURRENT SQL_CCFLAGS,
    CURRENT TEMPORAL BUSINESS_TIME,
    CURRENT TEMPORAL SYSTEM_TIME,
    CURRENT TIME,
    CURRENT TIMESTAMP,
    CURRENT TIMEZONE,
    CURRENT USER,
    SESSION_USER,
    SYSTEM_USER,
    USER
);
