REASSIGN OWNED BY bob TO alice;

REASSIGN OWNED BY bob, ted TO alice;

REASSIGN OWNED BY bob, CURRENT_ROLE, ted, CURRENT_USER, sam, SESSION_USER TO alice;

REASSIGN OWNED BY bob TO CURRENT_ROLE;

REASSIGN OWNED BY bob TO CURRENT_USER;

REASSIGN OWNED BY bob TO SESSION_USER;
