CREATE SCHEMA asdf;

CREATE SCHEMA IF NOT EXISTS asdf;

CREATE SCHEMA asdf AUTHORIZATION bob;

CREATE SCHEMA AUTHORIZATION bob;

CREATE SCHEMA IF NOT EXISTS asdf AUTHORIZATION bob;

CREATE SCHEMA IF NOT EXISTS AUTHORIZATION bob;
