SELECT *
FROM TABX
WHERE
    MY_DT BETWEEN CURRENT_DATE - (DAYS(CURRENT_DATE) - 1) DAYS - 180 DAYS
    AND CURRENT_DATE;

SELECT CURRENT_DATE + COL1 YEAR
FROM TABY;

VALUES
NOW + 1 MICROSECOND,
NOW + 1 MICROSECONDS,
NOW - 1 SECOND,
NOW - 1 SECONDS,
NOW - 1 MINUTE,
NOW - 1 MINUTES,
NOW - 1 HOUR,
NOW + 1 HOURS,
NOW - 1 DAY,
CURRENT_DATE - 1 DAYS,
NOW - 1 MONTH,
NOW - 1 MONTHS,
NOW() - 1 YEAR,
NOW - 1 YEARS,
NOW - 1 DAY - 2 HOURS,
(NOW - 1 DAY) - 2 HOURS,
NOW - (2 * 3) DAY - 2 HOURS,
NOW + .5 DAY;
