SELECT * FROM project-a.dataset-b.table-c JOIN dataset-c.table-d USING (a);

SELECT * FROM a-1a.b.c;
SELECT * FROM a-1.b.c;
SELECT * FROM project23-123.dataset7-b1.table-2c JOIN dataset12-c1.table-1d USING (a);

SELECT
   col1-col2 AS newcol1,
   col1 - col2 AS newcol2
FROM
  table-a123;
