CREATE USER user_name FOR LOGIN login_name;

CREATE USER user_name FROM LOGIN login_name;

CREATE USER [TestUser];

CREATE USER Barry WITH PASSWORD = 'sdjklalie8rew8337!$d';

CREATE USER [TestUser] FROM EXTERNAL PROVIDER;

CREATE USER RIGHTSHOLDER WITHOUT LOGIN;

CREATE USER RIGHTSHOLDER WITHOUT LOGIN
WITH DEFAULT_SCHEMA = Other;

CREATE USER [Barry]
WITH ALLOW_ENCRYPTED_VALUE_MODIFICATIONS = ON;

CREATE USER Wanida FOR LOGIN WanidaBenshoof
WITH DEFAULT_SCHEMA = Marketing;

CREATE USER Wanida FOR LOGIN WanidaSmith
WITH DEFAULT_SCHEMA = Tech,
ALLOW_ENCRYPTED_VALUE_MODIFICATIONS = ON;

CREATE USER JinghaoLiu FOR CERTIFICATE CarnationProduction50;

CREATE USER JinghaoLiu FOR ASYMMETRIC KEY CarnationProduction;

CREATE USER Barry
WITH PASSWORD = 'sdjklalie8rew8337!$d',
SID = 0x241C11948AEEB749B0D22646DB1A19F2;
