{% if True %}
SELECT 1
{% elif True %}
SELECT 10
{% elif True %}
SELECT 100
{% elif True %}
SELECT 1000
{% elif True %}
SELECT 10000
{% elif True %}
SELECT 100000
{% elif True %}
SELECT 1000000
{% elif True %}
SELECT 10000000
{% else %}
SELECT 100000000
{% endif %}
