-- Old-style vacuum commands

VACUUM;
VACUUM FULL;
VACUUM FREEZE;
VACUUM VERBOSE;
VACUUM ANALYZE;
VACUUM ANALYSE;
VACUUM FULL FREEZE VERBOSE ANALYSE;

VACUUM tbl;
VACUUM tbl1, tbl2;
VACUUM FULL FREEZE VERBOSE ANALYSE tbl1, tbl2;
VACUUM FULL tbl1 (col1, col2), tbl2;
VACUUM FULL tbl1 (col1), tbl2 (col1, col2);

-- New-style vacuum commands

VACUUM (FULL);
VACUUM (FULL, FREEZE) tbl1;
VACUUM (FULL, FREEZE) tbl1 (col1, col2), tbl2 (col3);
VACUUM (FULL TRUE, FREEZE);
VACUUM (
    FULL TRUE,
    FREEZE FALSE,
    VERBOSE,
    ANALYZE,
    ANALYSE,
    DISABLE_PAGE_SKIPPING,
    SKIP_LOCKED,
    INDEX_CLEANUP on,
    PROCESS_TOAST,
    TRUNCATE,
    PARALLEL 70
);
VACUUM (INDEX_CLEANUP off);
VACUUM (INDEX_CLEANUP auto);
