SELECT
  current_date AS col1,
  current_timestamp AS col2,
  current_time as col3,
  localtime as col4,
  localtimestamp as col5,
  current_role as col6,
  current_schema as col7,
  current_user as col8,
  session_user as col9,
  system_user as col10,
  user as col11
;
