
SELECT CAST(1 AS TINYINT);

SELECT CAST(2 AS SMALLINT);

-- Synonyms of INTEGER
-- https://trino.io/docs/422/language/types.html#integer-or-int
SELECT CAST(3 AS INTEGER);
SELECT CAST(4 AS INT);

SELECT CAST(5 AS BIGINT);
