CALL somefunction('a');
CALL somefunction(test('something'));
CALL somefunction('test', @test1, test2, test3('test'), "test4");
CALL somefunction();
CALL `somefunction`('a');
CALL testdb.testfunc(123);
