ALTER NETWORK POLICY mypolicy1 SET ALLOWED_IP_LIST=('192.168.1.0/24','192.168.255.100')
                                   BLOCKED_IP_LIST=('192.168.1.99');

ALTER NETWORK POLICY foo RENAME TO bar;

ALTER NETWORK POLICY foo SET TAG admin.my_tag = 'foo';

ALTER NETWORK POLICY foo UNSET TAG admin.my_tag;
--
ALTER NETWORK POLICY  IF EXISTS foo UNSET COMMENT;

ALTER NETWORK POLICY foo REMOVE ALLOWED_NETWORK_RULE_LIST='foo';

ALTER NETWORK POLICY foo ADD BLOCKED_NETWORK_RULE_LIST='bar';
