CREATE TEMPORARY TABLE tbl_name AS
    SELECT * FROM table_name;

CREATE TEMPORARY TABLE tbl_name
    SELECT * FROM table_name;

CREATE TEMPORARY TABLE `db_name`.tbl_name AS
    SELECT * FROM table_name;

CREATE TEMPORARY TABLE `db_name`.tbl_name
    SELECT * FROM table_name;

CREATE TEMPORARY TABLE IF NOT EXISTS `db_name`.tbl_name AS
    SELECT * FROM table_name;

CREATE TEMPORARY TABLE IF NOT EXISTS `db_name`.tbl_name
    SELECT * FROM table_name;

CREATE TEMPORARY TABLE IF NOT EXISTS tbl_name AS
    SELECT * FROM table_name;

CREATE TEMPORARY TABLE IF NOT EXISTS tbl_name
    SELECT * FROM table_name;
