DROP MACRO mcr;
DROP MACRO TABLE mt;
DROP FUNCTION mcr;
DROP FUNCTION TABLE mt;
DROP MACRO IF EXISTS mymacro;
DROP MACRO myschema.mymacro;
DROP MACRO "myschema".mymacro;
DROP MACRO mcr RESTRICT;
DROP MACRO mcr CASCADE;
