SELECT
    CURRENT_ACCOUNT() as CURRENT_ACCOUNT,
    CURRENT_ROLE() as CURRENT_ROLE,
    CURRENT_REGION() AS CURRENT_REGION,
    CURRENT_SESSION() as CURRENT_SESSION,
    CURRENT_USER() as CURRENT_USER,
    CURRENT_ACCOUNT_NAME() as CURRENT_ACCOUNT_NAME,
    CURRENT_ORGANIZATION_NAME() as CURRENT_ORGANIZATION_NAME
