CREATE OR REPLACE TEMPORARY TABLE `db_name`.tbl_name AS
    SELECT * FROM tbl;

CREATE OR REPLACE TEMPORARY TABLE `db_name`.tbl_name
    SELECT * FROM tbl;

CREATE OR REPLACE TEMPORARY TABLE tbl_name AS
    SELECT * FROM tbl;

CREATE OR REPLACE TEMPORARY TABLE tbl_name
    SELECT * FROM tbl;

CREATE OR REPLACE TEMPORARY TABLE IF NOT EXISTS `db_name`.tbl_name AS
    SELECT * FROM tbl;

CREATE OR REPLACE TEMPORARY TABLE IF NOT EXISTS `db_name`.tbl_name
    SELECT * FROM tbl;

CREATE OR REPLACE TEMPORARY TABLE IF NOT EXISTS tbl_name AS
    SELECT * FROM tbl;

CREATE OR REPLACE TEMPORARY TABLE IF NOT EXISTS tbl_name
    SELECT * FROM tbl;
--
-- CREATE OR REPLACE TEMPORARY TABLE `db_name`.tbl_name ENGINE=MyISAM AS
--     SELECT * FROM tbl;
--
-- CREATE OR REPLACE TEMPORARY TABLE tbl_name ENGINE=InnoDB
--     SELECT * FROM tbl;
