SELECT CAST(1 AS FLOAT64);
SELECT SAFE_CAST(1.0 AS INT64);
SELECT CAST(ARRAY["element1"] AS ARRAY<STRING>);
SELECT CAST(b'\x48\x65\x6c\x6c\x6f' AS STRING FORMAT 'ASCII') AS bytes_to_string;
SELECT CAST(
  TIMESTAMP '2008-12-25 00:00:00+00:00'
  AS STRING FORMAT 'YYYY-MM-DD HH24:MI:SS TZH:TZM') AS date_time_to_string;
