CREATE TABLE users (
    user_id INTEGER PRIMARY KEY AUTOINCREMENT,
    username TEXT NOT NULL UNIQUE,
    password TEXT NOT NULL COLLATE NOCASE,
    email TEXT NOT NULL UNIQUE
);

CREATE TABLE users (
    user_id INTEGER PRIMARY KEY ASC AUTOINCREMENT
);

CREATE TABLE users (
    user_id INTEGER PRIMARY KEY DESC AUTOINCREMENT
);

CREATE TABLE example (
    id INTEGER PRIMARY KEY,
    description CHARACTER VARYING(32) NOT NULL
);
