CREATE OR REPLACE TAG IF NOT EXISTS boo;

CREATE TAG cost_center COMMENT = 'cost_center tag';

CREATE OR REPLACE TAG IF NOT EXISTS DATA_CLASSIFICATION ALLOWED_VALUES 'RESTRICTED', 'CONFIDENTIAL', 'PII';
