SELECT *
FROM Sales.Customer TABLESAMPLE SYSTEM (10 PERCENT);

SELECT *
FROM Sales.Customer TABLESAMPLE (10 ROWS);

SELECT *
FROM Sales.Customer TABLESAMPLE (10);

SELECT *
FROM Sales.Customer TABLESAMPLE SYSTEM (10 ROWS) REPEATABLE (100);
