SELECT
    SEMI_STRUCTURED_OBJECT:ID_FIELD::VARCHAR AS ID_FIELD,
    SEMI_STRUCTURED_OBJECT:METADATA$FILENAME::VARCHAR AS METADATA$FILENAME,
    PARSE_JSON($1):FILE_CONTENT_KEY FILE_CONTENT_KEY,
    DATEADD(MS, PARSE_JSON($1):EVENT_TIME '1970-01-01') EVENT_TIME,
    parse_json($1):METADATA$FILENAME METADATA$FILENAME,
    DATEADD(MS, PARSE_JSON($1):METADATA$INSERTION_TIME,  '1970-01-01') METADATA$INSERTION_TIME,
    DATEADD(MS, PARSE_JSON($1):METADATA$LAST_EVENT_INSERTION_TIME, '1970-01-01') METADATA$LAST_EVENT_INSERTION_TIME
FROM @staging_area_root/ingest/data/file_name.ndjson
