ALTER TABLE mydataset.mytable
ALTER COLUMN IF EXISTS A SET OPTIONS (
        description='some description here'
),
ALTER COLUMN IF EXISTS B DROP NOT NULL,
ALTER COLUMN IF EXISTS C DROP DEFAULT,
ALTER COLUMN IF EXISTS D SET DATA TYPE FLOAT64,
ALTER COLUMN IF EXISTS E SET DEFAULT 0,
ALTER COLUMN IF EXISTS F SET DEFAULT CURRENT_TIMESTAMP()
;
