INSERT INTO kv VALUES ('A');

INSERT INTO kv (v) VALUES ('a');

INSERT INTO kv (k) VALUES ('nil1');

INSERT INTO kv (k) VALUES ('nil2');

INSERT INTO kv VALUES ('nil3', NULL);

INSERT INTO kv VALUES ('nil4', NULL);

INSERT INTO kv (k,v) VALUES ('a', 'b'), ('c', 'd');

-- RETURNING
INSERT INTO t (a) VALUES (10) RETURNING b;
INSERT INTO t VALUES (7, 8) RETURNING (SELECT 1);
INSERT INTO t VALUES (7, 8) RETURNING z;
