-- Simple LET
LET CUR_TABLE CURSOR FOR SELECT * FROM THE_TABLE;
-- Simple LOOP
BEGIN
    SELECT 1;
    SELECT 2;
END;
-- Simple FOR
FOR THE_ROW IN CUR_TABLE DO
    LET NEW_VAR := THE_ROW.COL1 || "_SUFFIX";
END FOR;
-- FOR and LET in LOOP
BEGIN
LET CUR_TABLE CURSOR FOR SELECT * FROM THE_TABLE;
FOR THE_ROW IN CUR_TABLE DO
    LET NEW_VAR := THE_ROW.COL1 || "_SUFFIX";
END FOR;
END;
