SHOW VOLUMES;

SHOW VOLUMES IN sampledb;

SHOW VOLUMES FROM sampledb;

SHOW VOLUMES LIKE 'regex*';

SHOW VOLUMES 'regex*';

SHOW VOLUMES IN sampledb LIKE 'regex*';

SHOW VOLUMES IN sampledb 'regex*';

SHOW VOLUMES FROM sampledb LIKE 'regex*';

SHOW VOLUMES FROM sampledb 'regex*';
