BEGIN TRANSACTION;
CREATE TABLE IF NOT EXISTS "tbl" ( "col" TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP );
COMMIT;

CREATE TABLE t (
	id TEXT DEFAULT datetime('now')
);

CREATE TABLE t (
	id TEXT DEFAULT (datetime('now'))
);

CREATE TABLE t (
	id TEXT DEFAULT ('now')
);

CREATE TABLE t (
	id TEXT DEFAULT 'now'
);

CREATE TABLE t (
	id TEXT DEFAULT -1
);
CREATE TABLE t (
	id TEXT DEFAULT +1
);
