ALTER ACCOUNT SET TIMEZONE = 'UTC';

ALTER ACCOUNT SET
  ALLOW_ID_TOKEN = TRUE,
  DEFAULT_DDL_COLLATION = 'en-ci',
  CLIENT_ENCRYPTION_KEY_SIZE = 128,
  NETWORK_POLICY = mypolicy
;

ALTER ACCOUNT UNSET TIMEZONE;
ALTER ACCOUNT UNSET DATA_RETENTION_TIME_IN_DAYS, JSON_INDENT;

ALTER ACCOUNT SET RESOURCE_MONITOR = VERY_RESTRICTIVE_MONITOR;

ALTER ACCOUNT SET PASSWORD POLICY mydb.security.at_least_twelve_characters;
ALTER ACCOUNT SET SESSION POLICY mydb.policies.only_one_hour;

ALTER ACCOUNT UNSET PASSWORD POLICY;
ALTER ACCOUNT UNSET SESSION POLICY;

ALTER ACCOUNT SET TAG
  env = 'prod',
  domain = 'sales'
;

ALTER ACCOUNT UNSET TAG env, domain;

ALTER ACCOUNT SET EVENT_TABLE = LOG_TRACE_DB.PUBLIC.EVENT_TABLE;
