SELECT CURRENT_DATE + INTERVAL '1' YEAR;
;
SELECT CURRENT_DATE - INTERVAL '14' MONTH;
;
SELECT CURRENT_DATE + INTERVAL '32' DAY;
;
SELECT CURRENT_TIME - INTERVAL '13' HOUR;
;
SELECT CURRENT_TIME + INTERVAL '64' MINUTE;
;
SELECT CURRENT_TIME - INTERVAL '61' SECOND;
