COMMENT ON CATALOG my_catalog IS 'This is my catalog';

COMMENT ON CONNECTION mysql_connection IS 'this is a mysql connection';

COMMENT ON SCHEMA my_schema IS 'This is my schema';

COMMENT ON DATABASE my_other_schema IS 'This is my other schema';

COMMENT ON TABLE my_table IS 'This is my table';

COMMENT ON TABLE my_table IS NULL;

COMMENT ON SHARE my_share IS 'A good share';

COMMENT ON RECIPIENT my_recipient IS 'A good recipient';

COMMENT ON PROVIDER my_provider IS 'A good provider';

COMMENT ON VOLUME my_volume IS 'Huge volume';
