SET LOCAL search_path = DEFAULT;
SET search_path TO my_schema, public;
SET datestyle TO postgres, dmy;
SET SESSION datestyle TO postgres, 'dmy';
SET value = on, off, auto;
SET value = TRUE, FALSE;
SET TIME ZONE 'PST8PDT';
SET TIME ZONE 'Europe/Rome';
SET TIME ZONE LOCAL;
SET TIME ZONE DEFAULT;
SET SCHEMA  'my_schema';
SET SCHEMA  'public';
SET ROLE my_role;
SET ROLE "my role";
SET ROLE NONE;
SET LOCAL search_path = schema_name;
SET LOCAL search_path = "schema_name";
