-- Databricks notebook source
-- MAGIC %md
-- MAGIC # Dummy Notebook

-- COMMAND ----------

-- DBTITLE 1,Select Data

SELECT x FROM y

-- COMMAND ----------

-- MAGIC %python
-- MAGIC foo = 'bar'
-- MAGIC print(foo)

-- COMMAND ----------

SELECT a FROM b;

-- COMMAND ----------

-- MAGIC %sh
-- MAGIC echo heloworld
