DROP OWNED BY bob;

DROP OWNED BY bob, alice;

DROP OWNED BY CURRENT_ROLE;

DROP OWNED BY CURRENT_USER;

DROP OWNED BY SESSION_USER;

DROP OWNED BY bob, CURRENT_ROLE, alice, CURRENT_USER, ted;

DROP OWNED BY bob CASCADE;

DROP OWNED BY bob RESTRICT;

DROP OWNED BY bob, alice CASCADE;

DROP OWNED BY bob, alice RESTRICT;
