SET DATESTYLE TO German;
SET ESCAPE_STRING_WARNING TO OFF;
SET LOCALE TO en_GB;
SET ROLE applogs;
SET SEARCH_PATH TO store, public;
SET SESSION AUTHORIZATION debuguser;
SET SESSION AUTOCOMMIT TO on;
SET SESSION MEMORYCAP '2G';
SET SESSION MULTIPLEACTIVERESULTSETS TO on;
SET SESSION RESOURCE_POOL = ceo_pool;
SET SESSION RUNTIMECAP '10 minutes';
SET SESSION TEMPSPACECAP '20G';
SET SESSION WORKLOAD analytics;
SET STANDARD_CONFORMING_STRINGS TO OFF;
SET TIME ZONE TO DEFAULT;
