DROP ROW ACCESS POLICY IF EXISTS example_policy_name ON example_dataset.example_table;

DROP CAPACITY `example-project.region-us.example_commitment`;

DROP RESERVATION IF EXISTS `example-project.region-us.example_reservation`;

DROP ASSIGNMENT `example-project.region-us.example_reservation.example_assignment`;

DROP SEARCH INDEX IF EXISTS example_index ON example_dataset.example_table;

DROP VECTOR INDEX example_index ON example_dataset.example_table;
